@@ -129,6 +129,21 @@ brasero_burn_caps_get_blanking_flags_real (BraseroBurnCaps *caps,
 		return BRASERO_BURN_NOT_SUPPORTED;
 	}
 
+	/* This is a special case that is in MMC specs:
+	 * DVD-RW sequential must be fully blanked
+	 * if we really want multisession support. */
+	if (BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW)
+	&& (session_flags & BRASERO_BURN_FLAG_MULTI)) {
+		if (compulsory_flags & BRASERO_BURN_FLAG_FAST_BLANK) {
+			BRASERO_BURN_LOG ("fast media blanking only supported but multisession required for DVDRW");
+			return BRASERO_BURN_NOT_SUPPORTED;
+		}
+
+		supported_flags &= ~BRASERO_BURN_FLAG_FAST_BLANK;
+
+		BRASERO_BURN_LOG ("removed fast blank for a DVDRW with multisession");
+	}
+
 	if (supported)
 		*supported = supported_flags;
 	if (compulsory)
@@ -137,6 +152,23 @@ brasero_burn_caps_get_blanking_flags_real (BraseroBurnCaps *caps,
 	return BRASERO_BURN_OK;
 }
 
+/**
+ * brasero_burn_session_get_blank_flags:
+ * @session: a #BraseroBurnSession
+ * @supported: a #BraseroBurnFlag
+ * @compulsory: a #BraseroBurnFlag
+ *
+ * Given the various parameters stored in @session,
+ * stores in @supported and @compulsory, the flags
+ * that can be used (@supported) and must be used
+ * (@compulsory) when blanking the medium in the
+ * #BraseroDrive (set with brasero_burn_session_set_burner ()).
+ *
+ * Return value: a #BraseroBurnResult.
+ * BRASERO_BURN_OK if the retrieval was successful.
+ * BRASERO_BURN_ERR otherwise.
+ **/
+
 BraseroBurnResult
 brasero_burn_session_get_blank_flags (BraseroBurnSession *session,
 				      BraseroBurnFlag *supported,
@@ -185,6 +217,16 @@ brasero_burn_caps_can_blank_real (BraseroBurnCaps *self,
 		return BRASERO_BURN_NOT_SUPPORTED;
 	}
 
+	/* This is a special case from MMC: DVD-RW sequential
+	 * can only be multisession is they were fully blanked
+	 * so if there are the two flags, abort. */
+	if (BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW)
+	&&  (flags & BRASERO_BURN_FLAG_MULTI)
+	&&  (flags & BRASERO_BURN_FLAG_FAST_BLANK)) {
+		BRASERO_BURN_LOG ("fast media blanking only supported but multisession required for DVDRW");
+		return BRASERO_BURN_NOT_SUPPORTED;
+	}
+
 	for (iter = self->priv->caps_list; iter; iter = iter->next) {
 		BraseroCaps *caps;
 		GSList *links;
@@ -232,6 +274,20 @@ brasero_burn_caps_can_blank_real (BraseroBurnCaps *self,
 	return BRASERO_BURN_NOT_SUPPORTED;
 }
 
+/**
+ * brasero_burn_session_can_blank:
+ * @session: a #BraseroBurnSession
+ *
+ * Given the various parameters stored in @session, this
+ * function checks whether the medium in the
+ * #BraseroDrive (set with brasero_burn_session_set_burner ())
+ * can be blanked.
+ *
+ * Return value: a #BraseroBurnResult.
+ * BRASERO_BURN_OK if it is possible.
+ * BRASERO_BURN_ERR otherwise.
+ **/
+
 BraseroBurnResult
 brasero_burn_session_can_blank (BraseroBurnSession *session)
 {
@@ -258,10 +314,6 @@ brasero_burn_session_can_blank (BraseroBurnSession *session)
 	return result;
 }
 
-/**
- *
- */
-
 static void
 brasero_caps_link_get_record_flags (BraseroCapsLink *link,
 				    BraseroMedia media,
@@ -602,6 +654,24 @@ brasero_caps_try_output_with_blanking (BraseroBurnCaps *self,
 				       io_flags);
 }
 
+/**
+ * brasero_burn_session_input_supported:
+ * @session: a #BraseroBurnSession
+ * @input: a #BraseroTrackType
+ * @use_flags: a #gboolean
+ *
+ * Given the various parameters stored in @session, this
+ * function checks whether a session with the data type
+ * @type could be burnt to the medium in the #BraseroDrive (set 
+ * through brasero_burn_session_set_burner ()).
+ * If @use_flags is TRUE, then flags are taken into account
+ * and are not if it is FALSE.
+ *
+ * Return value: a #BraseroBurnResult.
+ * BRASERO_BURN_OK if it is possible.
+ * BRASERO_BURN_ERR otherwise.
+ **/
+
 BraseroBurnResult
 brasero_burn_session_input_supported (BraseroBurnSession *session,
 				      BraseroTrackType *input,
@@ -661,6 +731,16 @@ brasero_burn_session_input_supported (BraseroBurnSession *session,
 	return BRASERO_BURN_OK;
 }
 
+/**
+ * brasero_burn_session_output_supported:
+ * @session: a #BraseroBurnSession *
+ * @output: a #BraseroTrackType *
+ *
+ * Make sure that the image type or medium type defined in @output can be
+ * created/burnt given the parameters and the current data set in @session.
+ *
+ * Return value: BRASERO_BURN_OK if the medium type or the image type can be used as an output.
+ **/
 BraseroBurnResult
 brasero_burn_session_output_supported (BraseroBurnSession *session,
 				       BraseroTrackType *output)
@@ -708,6 +788,13 @@ brasero_burn_session_output_supported (BraseroBurnSession *session,
 	return BRASERO_BURN_OK;
 }
 
+/* This function is not public API yet because it was too
+ * late; so use it internally for now. It's mainly for 
+ * convenience.*/
+gboolean
+brasero_drive_can_write_media (BraseroDrive *drive,
+                               BraseroMedia media);
+
 /**
  * This is only to be used in case one wants to copy using the same drive.
  * It determines the possible middle image type.
@@ -716,9 +803,11 @@ brasero_burn_session_output_supported (BraseroBurnSession *session,
 static BraseroBurnResult
 brasero_burn_caps_is_session_supported_same_src_dest (BraseroBurnCaps *self,
 						      BraseroBurnSession *session,
+                                                      BraseroTrackType *tmp_type,
 						      gboolean use_flags)
 {
 	GSList *iter;
+	BraseroDrive *burner;
 	BraseroTrackType input;
 	BraseroTrackType output;
 	BraseroImageFormat format;
@@ -749,6 +838,7 @@ brasero_burn_caps_is_session_supported_same_src_dest (BraseroBurnCaps *self,
 	/* Find one available output format */
 	format = BRASERO_IMAGE_FORMAT_CDRDAO;
 	output.type = BRASERO_TRACK_TYPE_IMAGE;
+	burner = brasero_burn_session_get_burner (session);
 
 	for (; format > BRASERO_IMAGE_FORMAT_NONE; format >>= 1) {
 		gboolean supported;
@@ -776,25 +866,68 @@ brasero_burn_caps_is_session_supported_same_src_dest (BraseroBurnCaps *self,
 			if (caps->type.type != BRASERO_TRACK_TYPE_DISC)
 				continue;
 
+			/* Make sure this is supported by the drive */
+			if (!brasero_drive_can_write_media (burner, caps->type.subtype.media))
+				continue;
+
 			result = brasero_caps_find_link (caps,
-							 TRUE,
+							 use_flags,
 							 session_flags,
 							 caps->type.subtype.media,
-							 &input,
+							 &output,
 							 BRASERO_PLUGIN_IO_ACCEPT_FILE);
 
 			BRASERO_BURN_LOG_DISC_TYPE (caps->type.subtype.media,
 						    "Tested medium (%s)",
 						    result ? "working":"not working");
 
-			if (result)
+			if (result) {
+				if (tmp_type) {
+					tmp_type->type = BRASERO_TRACK_TYPE_IMAGE;
+					tmp_type->subtype.img_format = format;
+				}
+					
 				return BRASERO_BURN_OK;
+			}
 		}
 	}
 
 	return BRASERO_BURN_NOT_SUPPORTED;
 }
 
+BraseroBurnResult
+brasero_burn_session_get_tmp_image_type_same_src_dest (BraseroBurnSession *session,
+                                                       BraseroTrackType *image_type)
+{
+	BraseroBurnResult result;
+	BraseroBurnCaps *self;
+
+	self = brasero_burn_caps_get_default ();
+	result = brasero_burn_caps_is_session_supported_same_src_dest (self,
+	                                                               session,
+	                                                               image_type,
+	                                                               TRUE);
+	g_object_unref (self);
+	return result;
+}
+
+/**
+ * brasero_burn_session_can_burn:
+ * @session: a #BraseroBurnSession
+ * @use_flags: a #gboolean
+ *
+ * Given the various parameters stored in @session, this
+ * function checks whether the data contained in @session
+ * can be burnt to the medium in the #BraseroDrive (set 
+ * through brasero_burn_session_set_burner ()).
+ * If @use_flags is set to TRUE the flags are taken into
+ * account, otherwise they are not.
+ *
+ * Return value: a #BraseroBurnResult.
+ * BRASERO_BURN_OK if it is possible.
+ * BRASERO_BURN_ERR otherwise.
+ **/
+
 BraseroBurnResult
 brasero_burn_session_can_burn (BraseroBurnSession *session,
 			       gboolean use_flags)
@@ -811,7 +944,7 @@ brasero_burn_session_can_burn (BraseroBurnSession *session,
 	if (brasero_burn_session_same_src_dest_drive (session)) {
 		BraseroBurnResult res;
 
-		res = brasero_burn_caps_is_session_supported_same_src_dest (self, session, use_flags);
+		res = brasero_burn_caps_is_session_supported_same_src_dest (self, session, NULL, use_flags);
 		g_object_unref (self);
 		return res;
 	}
@@ -872,6 +1005,16 @@ brasero_burn_session_can_burn (BraseroBurnSession *session,
 	return BRASERO_BURN_OK;
 }
 
+/**
+ * brasero_burn_session_get_required_media_type:
+ * @session: a #BraseroBurnSession
+ *
+ * Return the medium types that could be used to burn
+ * @session.
+ *
+ * Return value: a #BraseroMedia
+ **/
+
 BraseroMedia
 brasero_burn_session_get_required_media_type (BraseroBurnSession *session)
 {
@@ -945,6 +1088,17 @@ brasero_burn_session_get_required_media_type (BraseroBurnSession *session)
 	return required_media;
 }
 
+/**
+ * brasero_burn_session_get_possible_output_formats:
+ * @session: a #BraseroBurnSession
+ * @formats: a #BraseroImageFormat
+ *
+ * Returns the disc image types that could be set to create
+ * an image given the current state of @session.
+ *
+ * Return value: a #guint. The number of formats available.
+ **/
+
 guint
 brasero_burn_session_get_possible_output_formats (BraseroBurnSession *session,
 						  BraseroImageFormat *formats)
@@ -978,13 +1132,23 @@ brasero_burn_session_get_possible_output_formats (BraseroBurnSession *session,
 	return num;
 }
 
+/**
+ * brasero_burn_session_get_default_output_format:
+ * @session: a #BraseroBurnSession
+ *
+ * Returns the default disc image type that should be set to create
+ * an image given the current state of @session.
+ *
+ * Return value: a #BraseroImageFormat
+ **/
+
 BraseroImageFormat
 brasero_burn_session_get_default_output_format (BraseroBurnSession *session)
 {
 	BraseroBurnCaps *self;
-	BraseroTrackType source;
-	BraseroTrackType output;
 	BraseroBurnResult result;
+	BraseroTrackType source = { BRASERO_TRACK_TYPE_NONE, { 0, }};
+	BraseroTrackType output = { BRASERO_TRACK_TYPE_NONE, { 0, }};
 
 	self = brasero_burn_caps_get_default ();
 
@@ -1169,9 +1333,16 @@ brasero_caps_get_flags (BraseroCaps *caps,
 	return retval;
 }
 
-static BraseroBurnFlag
+/* FIXME: make this public */
+gboolean
+brasero_medium_can_use_sao (BraseroMedium *medium);
+gboolean
+brasero_medium_can_use_tao (BraseroMedium *medium);
+
+static void
 brasero_medium_supported_flags (BraseroMedium *medium,
-				BraseroBurnFlag flags)
+				BraseroBurnFlag *supported_flags,
+                                BraseroBurnFlag *compulsory_flags)
 {
 	BraseroMedia media;
 
@@ -1179,43 +1350,60 @@ brasero_medium_supported_flags (BraseroMedium *medium,
 
 	/* This is always FALSE */
 	if (media & BRASERO_MEDIUM_PLUS)
-		flags &= ~BRASERO_BURN_FLAG_DUMMY;
+		(*supported_flags) &= ~BRASERO_BURN_FLAG_DUMMY;
 
 	/* Simulation is only possible according to write modes. This mode is
 	 * mostly used by cdrecord/wodim for CLONE images. */
 	else if (media & BRASERO_MEDIUM_DVD) {
 		if (!brasero_medium_can_use_dummy_for_sao (medium))
-			flags &= ~BRASERO_BURN_FLAG_DUMMY;
+			(*supported_flags) &= ~BRASERO_BURN_FLAG_DUMMY;
 	}
-	else if (flags & BRASERO_BURN_FLAG_DAO) {
+	else if ((*supported_flags) & BRASERO_BURN_FLAG_DAO) {
 		if (!brasero_medium_can_use_dummy_for_sao (medium))
-			flags &= ~BRASERO_BURN_FLAG_DUMMY;
+			(*supported_flags) &= ~BRASERO_BURN_FLAG_DUMMY;
 	}
 	else if (!brasero_medium_can_use_dummy_for_tao (medium))
-		flags &= ~BRASERO_BURN_FLAG_DUMMY;
+		(*supported_flags) &= ~BRASERO_BURN_FLAG_DUMMY;
+
+	/* The following is only true if we won't _have_ to blank
+	 * the disc since a CLOSED disc is not able for tao/sao.
+	 * so if BLANK_BEFORE_RIGHT is TRUE then we leave 
+	 * the benefit of the doubt, but flags should be rechecked
+	 * after the drive was blanked. */
+	if (((*compulsory_flags) & BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE) == 0
+	&&  !BRASERO_MEDIUM_RANDOM_WRITABLE (media)
+	&&  !brasero_medium_can_use_tao (medium)) {
+		(*supported_flags) &= ~BRASERO_BURN_FLAG_MULTI;
+
+		if (brasero_medium_can_use_sao (medium))
+			(*compulsory_flags) |= BRASERO_BURN_FLAG_DAO;
+		else
+			(*supported_flags) &= ~BRASERO_BURN_FLAG_DAO;
+	}
 
 	if (!brasero_medium_can_use_burnfree (medium))
-		flags &= ~BRASERO_BURN_FLAG_BURNPROOF;
-
-	return flags;
+		(*supported_flags) &= ~BRASERO_BURN_FLAG_BURNPROOF;
 }
 
-static BraseroBurnFlag
-brasero_burn_caps_flags_update_for_drive (BraseroBurnFlag flags,
-					  BraseroBurnSession *session)
+static void
+brasero_burn_caps_flags_update_for_drive (BraseroBurnSession *session,
+                                          BraseroBurnFlag *supported_flags,
+                                          BraseroBurnFlag *compulsory_flags)
 {
 	BraseroDrive *drive;
 	BraseroMedium *medium;
 
 	drive = brasero_burn_session_get_burner (session);
 	if (!drive)
-		return flags;
+		return;
 
 	medium = brasero_drive_get_medium (drive);
 	if (!medium)
-		return TRUE;
+		return;
 
-	return brasero_medium_supported_flags (medium, flags);
+	brasero_medium_supported_flags (medium,
+	                                supported_flags,
+	                                compulsory_flags);
 }
 
 static BraseroBurnResult
@@ -1312,13 +1500,12 @@ brasero_burn_caps_get_flags_for_medium (BraseroBurnCaps *self,
 	/* see if we can add BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE. Add it when:
 	 * - media can be blanked, it has audio or data and we're not merging
 	 * - media is not formatted and it can be blanked/formatted */
-	if (BRASERO_BURN_CAPS_SHOULD_BLANK (media, session_flags)
-	&&  brasero_burn_caps_can_blank_real (self, media, session_flags) == BRASERO_BURN_OK)
+	if (brasero_burn_caps_can_blank_real (self, media, session_flags) == BRASERO_BURN_OK)
 		(*supported_flags) |= BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE;
 	else if (session_flags & BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE)
 		return BRASERO_BURN_NOT_SUPPORTED;
 
-	if (((*supported_flags) & BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE)) {
+	if (((*supported_flags) & BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE) != 0) {
 		BraseroBurnFlag blank_compulsory = BRASERO_BURN_FLAG_NONE;
 		BraseroBurnFlag blank_supported = BRASERO_BURN_FLAG_NONE;
 
@@ -1342,8 +1529,6 @@ brasero_burn_caps_get_flags_for_medium (BraseroBurnCaps *self,
 		 * Ex: a CDRW with data appendable can be either appended (then
 		 * no DAO possible) or blanked and written (DAO possible). */
 
-		(*supported_flags) |= BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE;
-
 		/* result here is the result of the first operation, so if it
 		 * failed, BLANK before becomes compulsory. */
 		if (result != BRASERO_BURN_OK)
@@ -1382,6 +1567,24 @@ brasero_burn_caps_get_flags_for_medium (BraseroBurnCaps *self,
 	else if (result != BRASERO_BURN_OK)
 		return result;
 
+	/* These are a special case for DVDRW sequential */
+	if (BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW)) {
+		/* That's a way to give priority to MULTI over FAST
+		 * and leave the possibility to always use MULTI. */
+		if (session_flags & BRASERO_BURN_FLAG_MULTI)
+			(*supported_flags) &= ~BRASERO_BURN_FLAG_FAST_BLANK;
+		else if ((session_flags & BRASERO_BURN_FLAG_FAST_BLANK)
+		         &&  (session_flags & BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE)) {
+			/* We should be able to handle this case differently but unfortunately
+			 * there are buggy firmwares that won't report properly the supported
+			 * mode writes */
+			if (!((*supported_flags) & BRASERO_BURN_FLAG_DAO))
+					 return BRASERO_BURN_NOT_SUPPORTED;
+
+			(*compulsory_flags) |= BRASERO_BURN_FLAG_DAO;
+		}
+	}
+
 	if (session_flags & BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE) {
 		/* make sure we remove MERGE/APPEND from supported and
 		 * compulsory since that's not possible anymore */
@@ -1501,6 +1704,23 @@ brasero_burn_caps_get_flags_same_src_dest (BraseroBurnCaps *self,
 	return BRASERO_BURN_OK;
 }
 
+/**
+ * brasero_burn_session_get_burn_flags:
+ * @session: a #BraseroBurnSession
+ * @supported: a #BraseroBurnFlag or NULL
+ * @compulsory: a #BraseroBurnFlag or NULL
+ *
+ * Given the various parameters stored in @session, this function
+ * stores:
+ * - the flags that can be used (@supported)
+ * - the flags that must be used (@compulsory)
+ * when writing @session to a disc.
+ *
+ * Return value: a #BraseroBurnResult.
+ * BRASERO_BURN_OK if the retrieval was successful.
+ * BRASERO_BURN_ERR otherwise.
+ **/
+
 BraseroBurnResult
 brasero_burn_session_get_burn_flags (BraseroBurnSession *session,
 				     BraseroBurnFlag *supported,
@@ -1546,6 +1766,7 @@ brasero_burn_session_get_burn_flags (BraseroBurnSession *session,
 
 	/* special case */
 	if (brasero_burn_session_same_src_dest_drive (session)) {
+		BRASERO_BURN_LOG ("Same source and destination");
 		result = brasero_burn_caps_get_flags_same_src_dest (self,
 								    session,
 								    &supported_flags,
@@ -1606,8 +1827,9 @@ brasero_burn_session_get_burn_flags (BraseroBurnSession *session,
 	if (result != BRASERO_BURN_OK)
 		return result;
 
-	supported_flags = brasero_burn_caps_flags_update_for_drive (supported_flags,
-								    session);
+	brasero_burn_caps_flags_update_for_drive (session,
+	                                          &supported_flags,
+	                                          &compulsory_flags);
 
 	if (supported)
 		*supported = supported_flags;
