@@ -563,13 +563,78 @@ brasero_session_cfg_save_drive_properties (BraseroSessionCfg *self,
 	g_object_unref (client);
 }
 
+static void
+brasero_session_cfg_set_drive_properties_default_flags (BraseroSessionCfg *self)
+{
+	BraseroMedia media;
+	BraseroSessionCfgPrivate *priv;
+
+	priv = BRASERO_SESSION_CFG_PRIVATE (self);
+
+	media = brasero_burn_session_get_dest_media (BRASERO_BURN_SESSION (self));
+
+	if (BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_PLUS)
+	||  BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_RESTRICTED)
+	||  BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_PLUS_DL)) {
+		/* This is a special case to favour libburn/growisofs
+		 * wodim/cdrecord for these types of media. */
+		if (priv->supported & BRASERO_BURN_FLAG_MULTI) {
+			brasero_burn_session_add_flag (BRASERO_BURN_SESSION (self),
+						       BRASERO_BURN_FLAG_MULTI);
+
+			priv->supported = BRASERO_BURN_FLAG_NONE;
+			priv->compulsory = BRASERO_BURN_FLAG_NONE;
+			brasero_burn_session_get_burn_flags (BRASERO_BURN_SESSION (self),
+							     &priv->supported,
+							     &priv->compulsory);
+		}
+	}
+
+	/* Always set this flag whenever possible */
+	if (priv->supported & BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE) {
+		brasero_burn_session_add_flag (BRASERO_BURN_SESSION (self),
+					       BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE);
+
+		if (priv->supported & BRASERO_BURN_FLAG_FAST_BLANK
+		&& (media & BRASERO_MEDIUM_UNFORMATTED) == 0)
+			brasero_burn_session_add_flag (BRASERO_BURN_SESSION (self),
+						       BRASERO_BURN_FLAG_FAST_BLANK);
+
+		priv->supported = BRASERO_BURN_FLAG_NONE;
+		priv->compulsory = BRASERO_BURN_FLAG_NONE;
+		brasero_burn_session_get_burn_flags (BRASERO_BURN_SESSION (self),
+						     &priv->supported,
+						     &priv->compulsory);
+	}
+
+	/* When copying with same drive don't set write mode, it'll be set later */
+	if (!brasero_burn_session_same_src_dest_drive (BRASERO_BURN_SESSION (self))
+	&&  !(media & BRASERO_MEDIUM_DVD)) {
+		/* use DAO whenever it's possible except for DVDs otherwise
+		 * wodime which claims to support it will be used by default
+		 * instead of say growisofs. */
+		if (priv->supported & BRASERO_BURN_FLAG_DAO) {
+			brasero_burn_session_add_flag (BRASERO_BURN_SESSION (self), BRASERO_BURN_FLAG_DAO);
+
+			priv->supported = BRASERO_BURN_FLAG_NONE;
+			priv->compulsory = BRASERO_BURN_FLAG_NONE;
+			brasero_burn_session_get_burn_flags (BRASERO_BURN_SESSION (self),
+							     &priv->supported,
+							     &priv->compulsory);
+
+			/* NOTE: after setting DAO, some flags may become
+			 * compulsory like BLANK_BEFORE for CDRW with data */
+		}
+	}
+}
+
 static void
 brasero_session_cfg_set_drive_properties_flags (BraseroSessionCfg *self,
 						BraseroBurnFlag flags)
 {
+	BraseroDrive *drive;
 	BraseroMedia media;
 	BraseroBurnFlag flag;
-	BraseroDrive *drive;
 	BraseroMedium *medium;
 	BraseroBurnResult result;
 	BraseroBurnFlag original_flags;
@@ -641,59 +706,7 @@ brasero_session_cfg_set_drive_properties_flags (BraseroSessionCfg *self,
 		}
 	}
 
-	if (BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_PLUS)
-	||  BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_RESTRICTED)
-	||  BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_PLUS_DL)) {
-		/* This is a special case to favour libburn/growisofs
-		 * wodim/cdrecord for these types of media. */
-		if (priv->supported & BRASERO_BURN_FLAG_MULTI) {
-			brasero_burn_session_add_flag (BRASERO_BURN_SESSION (self),
-						       BRASERO_BURN_FLAG_MULTI);
-
-			priv->supported = BRASERO_BURN_FLAG_NONE;
-			priv->compulsory = BRASERO_BURN_FLAG_NONE;
-			brasero_burn_session_get_burn_flags (BRASERO_BURN_SESSION (self),
-							     &priv->supported,
-							     &priv->compulsory);
-		}
-	}
-
-	/* Always set this flag whenever possible */
-	if (priv->supported & BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE) {
-		brasero_burn_session_add_flag (BRASERO_BURN_SESSION (self),
-					       BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE);
-
-		if (priv->supported & BRASERO_BURN_FLAG_FAST_BLANK
-		&& (media & BRASERO_MEDIUM_UNFORMATTED) == 0)
-			brasero_burn_session_add_flag (BRASERO_BURN_SESSION (self),
-						       BRASERO_BURN_FLAG_FAST_BLANK);
-
-		priv->supported = BRASERO_BURN_FLAG_NONE;
-		priv->compulsory = BRASERO_BURN_FLAG_NONE;
-		brasero_burn_session_get_burn_flags (BRASERO_BURN_SESSION (self),
-						     &priv->supported,
-						     &priv->compulsory);
-	}
-
-	/* When copying with same drive don't set write mode, it'll be set later */
-	if (!brasero_burn_session_same_src_dest_drive (BRASERO_BURN_SESSION (self))
-	&&  !(media & BRASERO_MEDIUM_DVD)) {
-		/* use DAO whenever it's possible except for DVDs otherwise
-		 * wodime which claims to support it will be used by default
-		 * instead of say growisofs. */
-		if (priv->supported & BRASERO_BURN_FLAG_DAO) {
-			brasero_burn_session_add_flag (BRASERO_BURN_SESSION (self), BRASERO_BURN_FLAG_DAO);
-
-			priv->supported = BRASERO_BURN_FLAG_NONE;
-			priv->compulsory = BRASERO_BURN_FLAG_NONE;
-			brasero_burn_session_get_burn_flags (BRASERO_BURN_SESSION (self),
-							     &priv->supported,
-							     &priv->compulsory);
-
-			/* NOTE: after setting DAO, some flags may become
-			 * compulsory like BLANK_BEFORE for CDRW with data */
-		}
-	}
+	brasero_session_cfg_set_drive_properties_default_flags (self);
 
 	/* allow flag changed signal again */
 	priv->inhibit_flag_sig = FALSE;
@@ -1081,61 +1094,105 @@ brasero_session_cfg_update (BraseroSessionCfg *self,
 	}
 
 	/* Check that current input and output work */
-	if (priv->CD_TEXT_modified) {
-		/* Try to redo what we undid (after all a new plugin could have
-		 * been activated in the mean time ...) and see what happens */
-		brasero_track_type_set_image_format (source,
-						     BRASERO_METADATA_INFO|
-						     brasero_track_type_get_image_format (source));
-		result = brasero_burn_session_input_supported (BRASERO_BURN_SESSION (self),
-							       source,
-							       FALSE);
-		if (result == BRASERO_BURN_OK) {
-			priv->CD_TEXT_modified = FALSE;
-
-			priv->configuring = TRUE;
-			brasero_session_cfg_set_tracks_audio_format (BRASERO_BURN_SESSION (self),
-								     brasero_track_type_get_stream_format (source));
-			priv->configuring = FALSE;
-		}
-		else {
-			/* No, nothing's changed */
+	if (brasero_track_type_get_has_stream (source)) {
+		if (priv->CD_TEXT_modified) {
+			/* Try to redo what we undid (after all a new plugin could have
+			 * been activated in the mean time ...) and see what happens */
 			brasero_track_type_set_stream_format (source,
-							      (~BRASERO_METADATA_INFO) &
+							      BRASERO_METADATA_INFO|
 							      brasero_track_type_get_stream_format (source));
 			result = brasero_burn_session_input_supported (BRASERO_BURN_SESSION (self),
 								       source,
 								       FALSE);
-		}
-	}
-	else {
-		/* Don't use flags as they'll be adapted later. */
-		result = brasero_burn_session_can_burn (BRASERO_BURN_SESSION (self),
-							FALSE);
-		if (result != BRASERO_BURN_OK
-		&&  brasero_track_type_get_has_stream (source)
-		&& (brasero_track_type_get_stream_format (source) & BRASERO_METADATA_INFO)) {
-			/* Another special case in case some burning backends 
-			 * don't support CD-TEXT for audio (libburn). If no
-			 * other backend is available remove CD-TEXT option but
-			 * tell user... */
-			brasero_track_type_set_stream_format (source,
-							      (~BRASERO_METADATA_INFO) &
-							      brasero_track_type_get_stream_format (source));
-
-			result = brasero_burn_session_input_supported (BRASERO_BURN_SESSION (self),
-								       source,
-								       FALSE);
-			BRASERO_BURN_LOG ("Tested support without Metadata information (result %d)", result);
 			if (result == BRASERO_BURN_OK) {
-				priv->CD_TEXT_modified = TRUE;
+				priv->CD_TEXT_modified = FALSE;
 
 				priv->configuring = TRUE;
 				brasero_session_cfg_set_tracks_audio_format (BRASERO_BURN_SESSION (self),
-									     brasero_track_type_get_has_stream (source));
+									     brasero_track_type_get_stream_format (source));
 				priv->configuring = FALSE;
 			}
+			else {
+				/* No, nothing's changed */
+				brasero_track_type_set_stream_format (source,
+								      (~BRASERO_METADATA_INFO) &
+								      brasero_track_type_get_stream_format (source));
+				result = brasero_burn_session_input_supported (BRASERO_BURN_SESSION (self),
+									       source,
+									       FALSE);
+			}
 		}
+		else {
+			result = brasero_burn_session_can_burn (BRASERO_BURN_SESSION (self), FALSE);
+
+			if (result != BRASERO_BURN_OK
+			&& (brasero_track_type_get_stream_format (source) & BRASERO_METADATA_INFO)) {
+				/* Another special case in case some burning backends 
+				 * don't support CD-TEXT for audio (libburn). If no
+				 * other backend is available remove CD-TEXT option but
+				 * tell user... */
+				brasero_track_type_set_stream_format (source,
+								      (~BRASERO_METADATA_INFO) &
+								      brasero_track_type_get_stream_format (source));
+
+				result = brasero_burn_session_input_supported (BRASERO_BURN_SESSION (self),
+									       source,
+									       FALSE);
+				BRASERO_BURN_LOG ("Tested support without Metadata information (result %d)", result);
+				if (result == BRASERO_BURN_OK) {
+					priv->CD_TEXT_modified = TRUE;
+
+					priv->configuring = TRUE;
+					brasero_session_cfg_set_tracks_audio_format (BRASERO_BURN_SESSION (self),
+										     brasero_track_type_get_has_stream (source));
+					priv->configuring = FALSE;
+				}
+			}
+		}
+	}
+	else if (brasero_track_type_get_has_medium (source)
+	&&  (brasero_track_type_get_medium_type (source) & BRASERO_MEDIUM_HAS_AUDIO)) {
+		BraseroImageFormat format = BRASERO_IMAGE_FORMAT_NONE;
+
+		/* If we copy an audio disc check the image
+		 * type we're writing to as it may mean that
+		 * CD-TEXT cannot be copied.
+		 * Make sure that the writing backend
+		 * supports writing CD-TEXT?
+		 * no, if a backend reports it supports an
+		 * image type it should be able to burn all
+		 * its data. */
+		if (!brasero_burn_session_is_dest_file (BRASERO_BURN_SESSION (self))) {
+			BraseroTrackType *tmp_type;
+
+			tmp_type = brasero_track_type_new ();
+
+			/* NOTE: this is the same as brasero_burn_session_can_burn () */
+			result = brasero_burn_session_get_tmp_image_type_same_src_dest (BRASERO_BURN_SESSION (self), tmp_type);
+			if (result == BRASERO_BURN_OK)
+				format = brasero_track_type_get_image_format (tmp_type);
+			else
+				format = BRASERO_IMAGE_FORMAT_NONE;
+
+			brasero_track_type_free (tmp_type);
+
+			BRASERO_BURN_LOG ("Temporary image type %i", format);
+		}
+		else {
+			result = brasero_burn_session_can_burn (BRASERO_BURN_SESSION (self),
+			                                        FALSE);
+			format = brasero_burn_session_get_output_format (BRASERO_BURN_SESSION (self));
+		}
+
+		priv->CD_TEXT_modified = FALSE;
+		if (!(format & (BRASERO_IMAGE_FORMAT_CDRDAO|BRASERO_IMAGE_FORMAT_CUE)))
+			priv->CD_TEXT_modified = TRUE;
+	}
+	else {
+		/* Don't use flags as they'll be adapted later. */
+		priv->CD_TEXT_modified = FALSE;
+		result = brasero_burn_session_can_burn (BRASERO_BURN_SESSION (self),
+							FALSE);
 	}
 
 	if (result != BRASERO_BURN_OK) {
@@ -1395,7 +1452,7 @@ brasero_session_cfg_flags_changed (BraseroBurnSession *session)
  * @cfg: a #BraseroSessionCfg
  * @flags: a #BraseroBurnFlag
  *
- * Adds all flags from @flags that are not supported.
+ * Adds all flags from @flags that are supported.
  *
  **/
 
@@ -1404,7 +1461,6 @@ brasero_session_cfg_add_flags (BraseroSessionCfg *self,
 			       BraseroBurnFlag flags)
 {
 	BraseroSessionCfgPrivate *priv;
-	BraseroDrive *drive;
 
 	priv = BRASERO_SESSION_CFG_PRIVATE (self);
 
@@ -1414,18 +1470,7 @@ brasero_session_cfg_add_flags (BraseroSessionCfg *self,
 	if ((brasero_burn_session_get_flags (BRASERO_BURN_SESSION (self)) & flags) == flags)
 		return;
 
-	brasero_burn_session_add_flag (BRASERO_BURN_SESSION (self), flags);
-	priv->supported = BRASERO_BURN_FLAG_NONE;
-	priv->compulsory = BRASERO_BURN_FLAG_NONE;
-	brasero_burn_session_get_burn_flags (BRASERO_BURN_SESSION (self),
-					     &priv->supported,
-					     &priv->compulsory);
-
-	/* Always save flags */
-	drive = brasero_burn_session_get_burner (BRASERO_BURN_SESSION (self));
-	if (drive && brasero_drive_get_medium (drive))
-		brasero_session_cfg_save_drive_flags (self, brasero_drive_get_medium (drive));
-
+	brasero_session_cfg_add_drive_properties_flags (self, flags);
 	brasero_session_cfg_update (self,
 				    FALSE,
 				    FALSE);
@@ -1450,11 +1495,12 @@ brasero_session_cfg_remove_flags (BraseroSessionCfg *self,
 	priv = BRASERO_SESSION_CFG_PRIVATE (self);
 
 	brasero_burn_session_remove_flag (BRASERO_BURN_SESSION (self), flags);
-	priv->supported = BRASERO_BURN_FLAG_NONE;
-	priv->compulsory = BRASERO_BURN_FLAG_NONE;
-	brasero_burn_session_get_burn_flags (BRASERO_BURN_SESSION (self),
-					     &priv->supported,
-					     &priv->compulsory);
+
+	/* For this case reset all flags as some flags could
+	 * be made available after the removal of one flag
+	 * Example: After the removal of MULTI, FAST_BLANK
+	 * becomes available again for DVDRW sequential */
+	brasero_session_cfg_set_drive_properties_default_flags (self);
 
 	/* Always save flags */
 	drive = brasero_burn_session_get_burner (BRASERO_BURN_SESSION (self));
