@@ -39,6 +39,9 @@
 
 #include <gconf/gconf-client.h>
 
+#include <gst/gst.h>
+#include <gst/pbutils/pbutils.h>
+
 #include "brasero-io.h"
 
 #include "burn-basics.h"
@@ -81,7 +84,7 @@ brasero_burn_action_to_string (BraseroBurnAction action)
 							N_("Preparing to write"),
 							N_("Writing leadin"),
 							N_("Writing CD-TEXT information"),
-							N_("Finalising"),
+							N_("Finalizing"),
 							N_("Writing leadout"),
 						        N_("Starting to record"),
 							N_("Success") };
@@ -201,15 +204,37 @@ brasero_caps_list_dump (void)
 }
 
 gboolean
-brasero_burn_library_start (void)
+brasero_burn_library_start (int *argc,
+                            char **argv [])
 {
 	GConfClient *client;
 
-	BRASERO_BURN_LOG ("Initializing Brasero-%i.%i.%i",
+	BRASERO_BURN_LOG ("Initializing Brasero-burn %i.%i.%i",
 			  BRASERO_MAJOR_VERSION,
 			  BRASERO_MINOR_VERSION,
 			  BRASERO_SUB);
 
+#if defined(HAVE_STRUCT_USCSI_CMD)
+	/* Work around: because on OpenSolaris brasero posiblely be run
+	 * as root for a user with 'Primary Administrator' profile,
+	 * a root dbus session will be autospawned at that time.
+	 * This fix is to work around
+	 * http://bugzilla.gnome.org/show_bug.cgi?id=526454
+	 */
+	g_setenv ("DBUS_SESSION_BUS_ADDRESS", "autolaunch:", TRUE);
+#endif
+
+	/* Initialize external libraries (threads... */
+	if (!g_thread_supported ())
+		g_thread_init (NULL);
+
+	/* ... and Gstreamer) */
+	if (!gst_init_check (argc, argv, NULL))
+		return FALSE;
+
+	/* This is for missing codec automatic install */
+	gst_pb_utils_init ();
+
 	/* preload some gconf keys */
 	client = gconf_client_get_default ();
 	gconf_client_add_dir (client,
@@ -234,6 +259,15 @@ brasero_burn_library_start (void)
 	return TRUE;
 }
 
+/**
+ * brasero_burn_library_get_plugins_list:
+ * 
+ * This function returns the list of plugins that 
+ * are available to libbrasero-burn.
+ *
+ * Returns: a #GSList that must be destroyed when not needed and each object unreffed.
+ **/
+
 GSList *
 brasero_burn_library_get_plugins_list (void)
 {
@@ -244,6 +278,8 @@ brasero_burn_library_get_plugins_list (void)
 void
 brasero_burn_library_stop (void)
 {
+	GConfClient *client;
+
 	if (plugin_manager) {
 		g_object_unref (plugin_manager);
 		plugin_manager = NULL;
@@ -257,8 +293,8 @@ brasero_burn_library_stop (void)
 	/* Cleanup the io thing */
 	brasero_io_shutdown ();
 
-	/* close HAL connection */
-//	brasero_hal_watch_destroy ();
+	client = gconf_client_get_default ();
+	gconf_client_remove_dir (client, "/apps/brasero", NULL);
 }
 
 gboolean
@@ -321,9 +357,14 @@ brasero_burn_library_input_supported (BraseroTrackType *type)
 }
 
 /**
+ * brasero_burn_library_get_media_capabilities:
+ * @media: a #BraseroMedia
+ *
  * Used to test what the library can do based on the medium type.
  * Returns BRASERO_MEDIUM_WRITABLE if the disc can be written
  * and / or BRASERO_MEDIUM_REWRITABLE if the disc can be erased.
+ *
+ * Returns: a #BraseroMedia
  */
 
 BraseroMedia
