@@ -35,6 +35,13 @@
 #include <unistd.h>
 #include <string.h>
 
+#ifdef HAVE_CAM_LIB_H
+#include <errno.h>
+#include <stdio.h>
+#include <fcntl.h>
+#include <camlib.h>
+#endif
+
 #include <glib.h>
 #include <glib-object.h>
 #include <glib/gi18n-lib.h>
@@ -63,17 +70,24 @@ struct _BraseroDrivePrivate
 	GDrive *gdrive;
 
 	GThread *probe;
+	GMutex *mutex;
+	GCond *cond;
 	gint probe_id;
 
 	BraseroMedium *medium;
 	BraseroDriveCaps caps;
 
-	gchar *path;
-	gchar *block_path;
+	gchar *udi;
+
+	gchar *name;
+
+	gchar *device;
+	gchar *block_device;
 
 	GCancellable *cancel;
 
 	guint probed:1;
+	guint has_medium:1;
 	guint probe_cancelled:1;
 };
 
@@ -88,6 +102,7 @@ static gulong drive_signals [LAST_SIGNAL] = {0, };
 
 enum {
 	PROP_NONE	= 0,
+	PROP_DEVICE,
 	PROP_GDRIVE,
 	PROP_UDI
 };
@@ -117,6 +132,9 @@ brasero_drive_get_gdrive (BraseroDrive *drive)
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
 
+	if (!priv->gdrive)
+		return NULL;
+
 	return g_object_ref (priv->gdrive);
 }
 
@@ -194,26 +212,67 @@ brasero_drive_eject (BraseroDrive *drive,
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
 
-	res = brasero_gio_operation_eject_drive (priv->gdrive,
-						 priv->cancel,
-						 wait,
-						 error);
-	if (res)
-		return TRUE;
+	/* reset if needed */
+	if (g_cancellable_is_cancelled (priv->cancel)) {
+		BRASERO_MEDIA_LOG ("Resetting GCancellable object");
+		g_cancellable_reset (priv->cancel);
+	}
+
+	BRASERO_MEDIA_LOG ("Trying to eject drive");
+	if (priv->gdrive) {
+		res = brasero_gio_operation_eject_drive (priv->gdrive,
+							 priv->cancel,
+							 wait,
+							 error);
+		if (res)
+			return TRUE;
+	}
+	else
+		BRASERO_MEDIA_LOG ("No GDrive");
 
 	if (!priv->medium)
 		return FALSE;
 
+	/* reset if needed */
+	if (g_cancellable_is_cancelled (priv->cancel)) {
+		BRASERO_MEDIA_LOG ("Resetting GCancellable object");
+		g_cancellable_reset (priv->cancel);
+	}
+
+	BRASERO_MEDIA_LOG ("Trying to eject volume");
 	gvolume = brasero_volume_get_gvolume (BRASERO_VOLUME (priv->medium));
-	res = brasero_gio_operation_eject_volume (gvolume,
-						  priv->cancel,
-						  wait,
-						  error);
-	g_object_unref (gvolume);
+	if (gvolume) {
+		res = brasero_gio_operation_eject_volume (gvolume,
+							  priv->cancel,
+							  wait,
+							  error);
+		g_object_unref (gvolume);
+	}
 
 	return res;
 }
 
+/**
+ * brasero_drive_cancel_current_operation:
+ * @drive: #BraseroDrive *
+ *
+ * Cancels all operations currently running for @drive
+ *
+ **/
+void
+brasero_drive_cancel_current_operation (BraseroDrive *drive)
+{
+	BraseroDrivePrivate *priv;
+
+	g_return_if_fail (drive != NULL);
+	g_return_if_fail (BRASERO_IS_DRIVE (drive));
+
+	priv = BRASERO_DRIVE_PRIVATE (drive);
+
+	BRASERO_MEDIA_LOG ("Cancelling GIO operation");
+	g_cancellable_cancel (priv->cancel);
+}
+
 /**
  * brasero_drive_get_bus_target_lun_string:
  * @drive: a #BraseroDrive
@@ -221,9 +280,13 @@ brasero_drive_eject (BraseroDrive *drive,
  * Returns the bus, target, lun ("{bus},{target},{lun}") as a string which is
  * sometimes needed by some backends like cdrecord.
  *
+ * NOTE: that function returns either bus/target/lun or the device path
+ * according to OSes. Basically it returns bus/target/lun only for FreeBSD
+ * which is the only OS in need for that. For all others it returns the device
+ * path. 
+ *
  * Return value: a string or NULL. The string must be freed when not needed
  *
- * Deprecated since 2.27.3
  **/
 gchar *
 brasero_drive_get_bus_target_lun_string (BraseroDrive *drive)
@@ -231,7 +294,7 @@ brasero_drive_get_bus_target_lun_string (BraseroDrive *drive)
 	g_return_val_if_fail (drive != NULL, NULL);
 	g_return_val_if_fail (BRASERO_IS_DRIVE (drive), NULL);
 
-	return NULL;
+	return brasero_device_get_bus_target_lun (brasero_drive_get_device (drive));
 }
 
 /**
@@ -252,7 +315,7 @@ brasero_drive_is_fake (BraseroDrive *drive)
 	g_return_val_if_fail (BRASERO_IS_DRIVE (drive), FALSE);
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
-	return (priv->gdrive == NULL);
+	return (priv->device == NULL);
 }
 
 /**
@@ -266,6 +329,7 @@ brasero_drive_is_fake (BraseroDrive *drive)
 gboolean
 brasero_drive_is_door_open (BraseroDrive *drive)
 {
+	const gchar *device;
 	BraseroDrivePrivate *priv;
 	BraseroDeviceHandle *handle;
 	BraseroScsiMechStatusHdr hdr;
@@ -274,10 +338,11 @@ brasero_drive_is_door_open (BraseroDrive *drive)
 	g_return_val_if_fail (BRASERO_IS_DRIVE (drive), FALSE);
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
-	if (!priv->gdrive)
+	if (!priv->device)
 		return FALSE;
 
-	handle = brasero_device_handle_open (priv->path, FALSE, NULL);
+	device = brasero_drive_get_device (drive);
+	handle = brasero_device_handle_open (device, FALSE, NULL);
 	if (!handle)
 		return FALSE;
 
@@ -309,7 +374,6 @@ brasero_drive_can_use_exclusively (BraseroDrive *drive)
 	g_return_val_if_fail (BRASERO_IS_DRIVE (drive), FALSE);
 
 	device = brasero_drive_get_device (drive);
-
 	handle = brasero_device_handle_open (device, TRUE, NULL);
 	if (!handle)
 		return FALSE;
@@ -342,7 +406,7 @@ brasero_drive_lock (BraseroDrive *drive,
 	g_return_val_if_fail (BRASERO_IS_DRIVE (drive), FALSE);
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
-	if (!priv->gdrive)
+	if (!priv->device)
 		return FALSE;
 
 	device = brasero_drive_get_device (drive);
@@ -357,6 +421,7 @@ brasero_drive_lock (BraseroDrive *drive,
 	else
 		BRASERO_MEDIA_LOG ("Device locked");
 
+	brasero_device_handle_close (handle);
 	return result;
 }
 
@@ -380,7 +445,7 @@ brasero_drive_unlock (BraseroDrive *drive)
 	g_return_val_if_fail (BRASERO_IS_DRIVE (drive), FALSE);
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
-	if (!priv->gdrive)
+	if (!priv->device)
 		return FALSE;
 
 	device = brasero_drive_get_device (drive);
@@ -395,6 +460,7 @@ brasero_drive_unlock (BraseroDrive *drive)
 	else
 		BRASERO_MEDIA_LOG ("Device unlocked");
 
+	brasero_device_handle_close (handle);
 	return result;
 }
 
@@ -416,14 +482,17 @@ brasero_drive_get_display_name (BraseroDrive *drive)
 	g_return_val_if_fail (BRASERO_IS_DRIVE (drive), NULL);
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
-	if (!priv->gdrive) {
+	if (!priv->device) {
 		/* Translators: This is a fake drive, a file, and means that
 		 * when we're writing, we're writing to a file and create an
 		 * image on the hard drive. */
 		return g_strdup (_("Image File"));
 	}
 
-	return g_drive_get_name (priv->gdrive);
+	if (priv->gdrive)
+		return g_drive_get_name (priv->gdrive);
+
+	return g_strdup (priv->name);
 }
 
 /**
@@ -432,7 +501,8 @@ brasero_drive_get_display_name (BraseroDrive *drive)
  *
  * Gets a string holding the device path for the drive.
  *
- * Return value: a string holding the device path
+ * Return value: a string holding the device path.
+ * On Solaris returns raw device.
  **/
 const gchar *
 brasero_drive_get_device (BraseroDrive *drive)
@@ -443,7 +513,7 @@ brasero_drive_get_device (BraseroDrive *drive)
 	g_return_val_if_fail (BRASERO_IS_DRIVE (drive), NULL);
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
-	return priv->path? priv->path:priv->block_path;
+	return priv->device;
 }
 
 /**
@@ -451,9 +521,15 @@ brasero_drive_get_device (BraseroDrive *drive)
  * @drive: a #BraseroDrive
  *
  * Gets a string holding the block device path for the drive. This can be used on
- * some other OS, like Solaris, for burning operations instead of the device
+ * some other OSes, like Solaris, for GIO operations instead of the device
  * path.
  *
+ * Solaris uses block device for GIO operations and
+ * uses raw device for system calls and backends
+ * like cdrtool.
+ *
+ * If such a path is not available, it returns the device path.
+ *
  * Return value: a string holding the block device path
  **/
 const gchar *
@@ -465,7 +541,7 @@ brasero_drive_get_block_device (BraseroDrive *drive)
 	g_return_val_if_fail (BRASERO_IS_DRIVE (drive), NULL);
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
-	return priv->block_path;
+	return priv->block_device? priv->block_device:priv->device;
 }
 
 /**
@@ -475,8 +551,7 @@ brasero_drive_get_block_device (BraseroDrive *drive)
  * Gets a string holding the HAL udi corresponding to this device. It can be used
  * to uniquely identify the drive.
  *
- * Return value: a string holding the HAL udi. Not to be freed
- * Deprecated since 2.27.3
+ * Return value: a string holding the HAL udi or NULL. Not to be freed
  **/
 const gchar *
 brasero_drive_get_udi (BraseroDrive *drive)
@@ -489,10 +564,14 @@ brasero_drive_get_udi (BraseroDrive *drive)
 	g_return_val_if_fail (BRASERO_IS_DRIVE (drive), NULL);
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
-	if (!priv->gdrive)
+	if (!priv->device || !priv->gdrive)
 		return NULL;
 
-	return g_drive_get_identifier (priv->gdrive, G_VOLUME_IDENTIFIER_KIND_HAL_UDI);
+	if (priv->udi)
+		return priv->udi;
+
+	priv->udi = g_drive_get_identifier (priv->gdrive, G_VOLUME_IDENTIFIER_KIND_HAL_UDI);
+	return priv->udi;
 }
 
 /**
@@ -515,8 +594,7 @@ brasero_drive_get_medium (BraseroDrive *drive)
 	g_return_val_if_fail (BRASERO_IS_DRIVE (drive), NULL);
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
-
-	if (!priv->probed && priv->gdrive)
+	if (!priv->probed)
 		return NULL;
 
 	return priv->medium;
@@ -542,6 +620,73 @@ brasero_drive_get_caps (BraseroDrive *drive)
 	return priv->caps;
 }
 
+/**
+ * brasero_drive_can_write_media:
+ * @drive: a #BraseroDrive
+ * @media: a #BraseroMedia
+ *
+ * Returns whether the disc can burn a specific media type.
+ *
+ * Return value: a #gboolean. TRUE if the drive can write this type of media and FALSE otherwise
+ **/
+gboolean
+brasero_drive_can_write_media (BraseroDrive *drive,
+                               BraseroMedia media)
+{
+	BraseroDrivePrivate *priv;
+
+	g_return_val_if_fail (drive != NULL, FALSE);
+	g_return_val_if_fail (BRASERO_IS_DRIVE (drive), FALSE);
+
+	priv = BRASERO_DRIVE_PRIVATE (drive);
+
+	if (!(media & BRASERO_MEDIUM_REWRITABLE)
+	&&   (media & BRASERO_MEDIUM_CLOSED))
+		return FALSE;
+
+	if (media & BRASERO_MEDIUM_FILE)
+		return FALSE;
+
+	if (BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_CDR))
+		return (priv->caps & BRASERO_DRIVE_CAPS_CDR) != 0;
+
+	if (BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDR))
+		return (priv->caps & BRASERO_DRIVE_CAPS_DVDR) != 0;
+
+	if (BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDR_PLUS))
+		return (priv->caps & BRASERO_DRIVE_CAPS_DVDR_PLUS) != 0;
+
+	if (BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_CDRW))
+		return (priv->caps & BRASERO_DRIVE_CAPS_CDRW) != 0;
+
+	if (BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW))
+		return (priv->caps & BRASERO_DRIVE_CAPS_DVDRW) != 0;
+
+	if (BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_RESTRICTED))
+		return (priv->caps & BRASERO_DRIVE_CAPS_DVDRW) != 0;
+
+	if (BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_PLUS))
+		return (priv->caps & BRASERO_DRIVE_CAPS_DVDRW_PLUS) != 0;
+
+	if (BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDR_PLUS_DL))
+		return (priv->caps & BRASERO_DRIVE_CAPS_DVDR_PLUS_DL) != 0;
+
+	if (BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_PLUS_DL))
+		return (priv->caps & BRASERO_DRIVE_CAPS_DVDRW_PLUS_DL) != 0;
+
+	if (BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVD_RAM))
+		return (priv->caps & BRASERO_DRIVE_CAPS_DVDRAM) != 0;
+
+	/* All types of BD-R */
+	if (BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_BD|BRASERO_MEDIUM_WRITABLE))
+		return (priv->caps & BRASERO_DRIVE_CAPS_BDR) != 0;
+
+	if (BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_BDRE))
+		return (priv->caps & BRASERO_DRIVE_CAPS_BDRW) != 0;
+
+	return FALSE;
+}
+
 /**
  * brasero_drive_can_write:
  * @drive: a #BraseroDrive
@@ -569,15 +714,6 @@ brasero_drive_can_write (BraseroDrive *drive)
 			      BRASERO_DRIVE_CAPS_DVDRW_PLUS_DL));
 }
 
-static void
-brasero_drive_init (BraseroDrive *object)
-{
-	BraseroDrivePrivate *priv;
-
-	priv = BRASERO_DRIVE_PRIVATE (object);
-	priv->cancel = g_cancellable_new ();
-}
-
 static void
 brasero_drive_medium_probed (BraseroMedium *medium,
 			     BraseroDrive *self)
@@ -589,6 +725,13 @@ brasero_drive_medium_probed (BraseroMedium *medium,
 	/* only when it is probed */
 	/* NOTE: BraseroMedium calls GDK_THREADS_ENTER/LEAVE() around g_signal_emit () */
 	priv->probed = TRUE;
+
+	if (brasero_medium_get_status (priv->medium) == BRASERO_MEDIUM_NONE) {
+		g_object_unref (priv->medium);
+		priv->medium = NULL;
+		return;
+	}
+
 	g_signal_emit (self,
 		       drive_signals [MEDIUM_INSERTED],
 		       0,
@@ -610,6 +753,208 @@ brasero_drive_probing (BraseroDrive *drive)
 	return priv->probed != TRUE;
 }
 
+static gboolean
+brasero_drive_probed_inside (gpointer data)
+{
+	BraseroDrive *self;
+	BraseroDrivePrivate *priv;
+
+	self = BRASERO_DRIVE (data);
+	priv = BRASERO_DRIVE_PRIVATE (self);
+
+	g_mutex_lock (priv->mutex);
+	priv->probe_id = 0;
+	g_mutex_unlock (priv->mutex);
+
+	if (priv->has_medium) {
+		if (priv->medium)
+			return FALSE;
+
+		BRASERO_MEDIA_LOG ("Medium inserted");
+
+		priv->medium = g_object_new (BRASERO_TYPE_VOLUME,
+					     "drive", self,
+					     NULL);
+
+		g_signal_connect (priv->medium,
+				  "probed",
+				  G_CALLBACK (brasero_drive_medium_probed),
+				  self);
+	}
+	else if (priv->medium) {
+		BraseroMedium *medium;
+
+		BRASERO_MEDIA_LOG ("Medium removed");
+
+		medium = priv->medium;
+		priv->medium = NULL;
+
+		g_signal_emit (self,
+			       drive_signals [MEDIUM_REMOVED],
+			       0,
+			       medium);
+
+		g_object_unref (medium);
+		priv->probed = TRUE;
+	}
+	else
+		priv->probed = TRUE;
+
+	return FALSE;
+}
+
+static gpointer
+brasero_drive_probe_inside_thread (gpointer data)
+{
+	gint counter = 0;
+	const gchar *device;
+	BraseroScsiErrCode code;
+	BraseroDrivePrivate *priv;
+	BraseroDeviceHandle *handle;
+	BraseroDrive *drive = BRASERO_DRIVE (data);
+
+	priv = BRASERO_DRIVE_PRIVATE (drive);
+
+	/* the drive might be busy (a burning is going on) so we don't block
+	 * but we re-try to open it every second */
+	device = brasero_drive_get_device (drive);
+	BRASERO_MEDIA_LOG ("Trying to open device %s", device);
+
+	handle = brasero_device_handle_open (device, FALSE, &code);
+	while (!handle && counter <= BRASERO_DRIVE_OPEN_ATTEMPTS) {
+		sleep (1);
+
+		if (priv->probe_cancelled) {
+			BRASERO_MEDIA_LOG ("Open () cancelled");
+			goto end;
+		}
+
+		counter ++;
+		handle = brasero_device_handle_open (device, FALSE, &code);
+	}
+
+	if (priv->probe_cancelled) {
+		BRASERO_MEDIA_LOG ("Open () cancelled");
+		goto end;
+	}
+
+	if (!handle) {
+		BRASERO_MEDIA_LOG ("Open () failed: medium busy");
+		goto end;
+	}
+
+	while (brasero_spc1_test_unit_ready (handle, &code) != BRASERO_SCSI_OK) {
+		if (code == BRASERO_SCSI_NO_MEDIUM) {
+			BRASERO_MEDIA_LOG ("No medium inserted");
+
+			priv->has_medium = FALSE;
+			goto end;
+		}
+
+		if (code != BRASERO_SCSI_NOT_READY) {
+			brasero_device_handle_close (handle);
+			BRASERO_MEDIA_LOG ("Device does not respond");
+			goto end;
+		}
+
+		sleep (2);
+
+		if (priv->probe_cancelled) {
+			brasero_device_handle_close (handle);
+			BRASERO_MEDIA_LOG ("Device probing cancelled");
+			goto end;
+		}
+	}
+
+	BRASERO_MEDIA_LOG ("Medium inserted");
+	brasero_device_handle_close (handle);
+
+	priv->has_medium = TRUE;
+
+end:
+
+	g_mutex_lock (priv->mutex);
+
+	if (!priv->probe_cancelled)
+		priv->probe_id = g_idle_add (brasero_drive_probed_inside, drive);
+
+	priv->probe = NULL;
+	g_cond_broadcast (priv->cond);
+	g_mutex_unlock (priv->mutex);
+
+	g_thread_exit (0);
+
+	return NULL;
+}
+
+static void
+brasero_drive_probe_inside (BraseroDrive *drive)
+{
+	BraseroDrivePrivate *priv;
+
+	priv = BRASERO_DRIVE_PRIVATE (drive);
+
+	priv->probed = FALSE;
+
+	/* Check that a probe is not already being performed */
+	g_mutex_lock (priv->mutex);
+	if (!priv->probe) {
+		BRASERO_MEDIA_LOG ("Setting new probe");
+
+		if (priv->probe_id) {
+			/* Remove the result reporting as
+			 * the status seem to have changed */
+			g_source_remove (priv->probe_id);
+			priv->probe_id = 0;
+		}
+
+		priv->probe = g_thread_create (brasero_drive_probe_inside_thread,
+					       drive,
+					       FALSE,
+					       NULL);
+	}
+	else
+		BRASERO_MEDIA_LOG ("Ongoing probe");
+	g_mutex_unlock (priv->mutex);
+}
+
+static void
+brasero_drive_medium_gdrive_changed_cb (BraseroDrive *gdrive,
+					BraseroDrive *drive)
+{
+	brasero_drive_probe_inside (drive);
+}
+
+static void
+brasero_drive_update_gdrive (BraseroDrive *drive,
+                             GDrive *gdrive)
+{
+	BraseroDrivePrivate *priv;
+
+	priv = BRASERO_DRIVE_PRIVATE (drive);
+	if (priv->gdrive) {
+		g_signal_handlers_disconnect_by_func (priv->gdrive,
+						      brasero_drive_medium_gdrive_changed_cb,
+						      drive);
+		g_object_unref (priv->gdrive);
+	}
+
+	BRASERO_MEDIA_LOG ("Setting GDrive %p", gdrive);
+
+	if (gdrive) {
+		priv->gdrive = g_object_ref (gdrive);
+
+		/* If it's not a fake drive then connect to signal for any
+		 * change and check medium inside */
+		g_signal_connect (priv->gdrive,
+				  "changed",
+				  G_CALLBACK (brasero_drive_medium_gdrive_changed_cb),
+				  drive);
+	}
+
+	brasero_drive_probe_inside (drive);
+}
+
 /**
  * brasero_drive_reprobe:
  * @drive: a #BraseroDrive
@@ -631,6 +976,12 @@ brasero_drive_reprobe (BraseroDrive *drive)
 	g_return_if_fail (BRASERO_IS_DRIVE (drive));
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
+
+	if (priv->gdrive) {
+		/* reprobe the contents of the drive system wide */
+		g_drive_poll_for_media (priv->gdrive, NULL, NULL, NULL);
+	}
+
 	if (!priv->medium)
 		return;
 
@@ -645,90 +996,23 @@ brasero_drive_reprobe (BraseroDrive *drive)
 		       0,
 		       medium);
 	g_object_unref (medium);
-	priv->probed = FALSE;
 
-	/* try to get a new one */
-	priv->medium = g_object_new (BRASERO_TYPE_VOLUME,
-				     "drive", drive,
-				     NULL);
-	g_signal_connect (priv->medium,
-			  "probed",
-			  G_CALLBACK (brasero_drive_medium_probed),
-			  drive);
-}
-
-static void
-brasero_drive_check_medium_inside_gdrive (BraseroDrive *self)
-{
-	BraseroDrivePrivate *priv;
-
-	priv = BRASERO_DRIVE_PRIVATE (self);
-
-	BRASERO_MEDIA_LOG ("Contents changed %i", g_drive_has_media (priv->gdrive));
-
-	if (g_drive_has_media (priv->gdrive)) {
-		if (priv->medium)
-			return;
-
-		BRASERO_MEDIA_LOG ("Medium inserted");
-
-		priv->probed = FALSE;
-		priv->medium = g_object_new (BRASERO_TYPE_VOLUME,
-					     "drive", self,
-					     NULL);
-
-		g_signal_connect (priv->medium,
-				  "probed",
-				  G_CALLBACK (brasero_drive_medium_probed),
-				  self);
-	}
-	else if (priv->medium) {
-		BraseroMedium *medium;
-
-		BRASERO_MEDIA_LOG ("Medium removed");
-
-		medium = priv->medium;
-		priv->medium = NULL;
-
-		g_signal_emit (self,
-			       drive_signals [MEDIUM_REMOVED],
-			       0,
-			       medium);
-		g_object_unref (medium);
-		priv->probed = TRUE;
-	}
-	else
-		priv->probed = TRUE;
-}
-
-static void
-brasero_drive_medium_gdrive_changed_cb (BraseroDrive *gdrive,
-					BraseroDrive *drive)
-{
-	brasero_drive_check_medium_inside_gdrive (drive);
+	brasero_drive_probe_inside (drive);
 }
 
 static gboolean
 brasero_drive_probed (gpointer data)
 {
-	BraseroDrive *drive = BRASERO_DRIVE (data);
 	BraseroDrivePrivate *priv;
 
 	priv = BRASERO_DRIVE_PRIVATE (data);
 
-	g_thread_join (priv->probe);
-	priv->probe = NULL;
-
-	/* If it's not a fake drive then connect to signal for any
-	 * change and check medium inside */
-	g_signal_connect (priv->gdrive,
-			  "changed",
-			  G_CALLBACK (brasero_drive_medium_gdrive_changed_cb),
-			  drive);
-
-	brasero_drive_check_medium_inside_gdrive (drive);
-
+	g_mutex_lock (priv->mutex);
 	priv->probe_id = 0;
+	g_mutex_unlock (priv->mutex);
+
+	brasero_drive_probe_inside (BRASERO_DRIVE (data));
+
 	return FALSE;
 }
 
@@ -758,6 +1042,8 @@ brasero_drive_get_caps_profiles (BraseroDrive *self,
 		return FALSE;
 	}
 
+	BRASERO_MEDIA_LOG ("Dectected medium is 0x%x", BRASERO_GET_16 (hdr->current_profile));
+
 	/* Go through all features available */
 	desc = hdr->desc;
 	profiles = (BraseroScsiProfileDesc *) desc->data;
@@ -857,78 +1143,150 @@ static gpointer
 brasero_drive_probe_thread (gpointer data)
 {
 	gint counter = 0;
-	const gchar *path;
+	const gchar *device;
+	BraseroScsiResult res;
+	BraseroScsiInquiry hdr;
 	BraseroScsiErrCode code;
 	BraseroDrivePrivate *priv;
 	BraseroDeviceHandle *handle;
 	BraseroDrive *drive = BRASERO_DRIVE (data);
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
-	path = brasero_drive_get_device (drive);
-	if (!path)
-		path = brasero_drive_get_block_device (drive);
 
 	/* the drive might be busy (a burning is going on) so we don't block
 	 * but we re-try to open it every second */
-	BRASERO_MEDIA_LOG ("Trying to open device %s", path);
+	device = brasero_drive_get_device (drive);
+	BRASERO_MEDIA_LOG ("Trying to open device %s", device);
 
-	handle = brasero_device_handle_open (path, FALSE, &code);
+	handle = brasero_device_handle_open (device, FALSE, &code);
 	while (!handle && counter <= BRASERO_DRIVE_OPEN_ATTEMPTS) {
 		sleep (1);
 
 		if (priv->probe_cancelled) {
 			BRASERO_MEDIA_LOG ("Open () cancelled");
-			priv->probe = NULL;
-			return NULL;
+			goto end;
 		}
 
 		counter ++;
-		handle = brasero_device_handle_open (path, FALSE, &code);
+		handle = brasero_device_handle_open (device, FALSE, &code);
 	}
 
 	if (priv->probe_cancelled) {
 		BRASERO_MEDIA_LOG ("Open () cancelled");
-		priv->probe = NULL;
-		return NULL;
+		goto end;
 	}
 
-	if (handle) {
-		BRASERO_MEDIA_LOG ("Open () succeeded");
-
-		if (!brasero_drive_get_caps_profiles (drive, handle, &code))
-			brasero_drive_get_caps_2A (drive, handle, &code);
-
-		brasero_device_handle_close (handle);
-
-		BRASERO_MEDIA_LOG ("Drive caps are %d", priv->caps);
-	}
-	else
+	if (!handle) {
 		BRASERO_MEDIA_LOG ("Open () failed: medium busy");
+		goto end;
+	}
+
+	while (brasero_spc1_test_unit_ready (handle, &code) != BRASERO_SCSI_OK) {
+		if (code == BRASERO_SCSI_NO_MEDIUM) {
+			BRASERO_MEDIA_LOG ("No medium inserted");
+			break;
+		}
+
+		if (code != BRASERO_SCSI_NOT_READY) {
+			brasero_device_handle_close (handle);
+			BRASERO_MEDIA_LOG ("Device does not respond");
+			goto end;
+		}
+
+		sleep (2);
+
+		if (priv->probe_cancelled) {
+			brasero_device_handle_close (handle);
+			BRASERO_MEDIA_LOG ("Device probing cancelled");
+			goto end;
+		}
+	}
+
+	BRASERO_MEDIA_LOG ("Device ready");
+
+	/* get additional information like the name */
+	res = brasero_spc1_inquiry (handle, &hdr, NULL);
+	if (res == BRASERO_SCSI_OK) {
+		gchar *name_utf8;
+		gchar *vendor;
+		gchar *model;
+		gchar *name;
+
+		vendor = g_strndup ((gchar *) hdr.vendor, sizeof (hdr.vendor));
+		model = g_strndup ((gchar *) hdr.name, sizeof (hdr.name));
+		name = g_strdup_printf ("%s %s", g_strstrip (vendor), g_strstrip (model));
+		g_free (vendor);
+		g_free (model);
+
+		/* make sure that's proper UTF-8 */
+		name_utf8 = g_convert_with_fallback (name,
+		                                     -1,
+		                                     "ASCII",
+		                                     "UTF-8",
+		                                     "_",
+		                                     NULL,
+		                                     NULL,
+		                                     NULL);
+		g_free (name);
+
+		priv->name = name_utf8;
+	}
+
+	/* Get supported medium types */
+	if (!brasero_drive_get_caps_profiles (drive, handle, &code))
+		brasero_drive_get_caps_2A (drive, handle, &code);
+
+	brasero_device_handle_close (handle);
+
+	BRASERO_MEDIA_LOG ("Drive caps are %d", priv->caps);
+
+end:
+
+	g_mutex_lock (priv->mutex);
+
+	if (!priv->probe_cancelled)
+		priv->probe_id = g_idle_add (brasero_drive_probed, drive);
+
+	priv->probe = NULL;
+	g_cond_broadcast (priv->cond);
+	g_mutex_unlock (priv->mutex);
+
+	g_thread_exit (0);
 
-	priv->probe_id = g_idle_add (brasero_drive_probed, drive);
 	return NULL;
 }
 
 static void
-brasero_drive_init_real (BraseroDrive *drive)
+brasero_drive_init_real_device (BraseroDrive *drive,
+                                const gchar *device)
 {
 	BraseroDrivePrivate *priv;
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
 
-	priv->block_path = g_drive_get_identifier (priv->gdrive, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
+#if defined(HAVE_STRUCT_USCSI_CMD)
+	/* On Solaris path points to raw device, block_path points to the block device. */
+	g_assert(g_str_has_prefix(device, "/dev/dsk/"));
+	priv->device = g_strdup_printf ("/dev/rdsk/%s", device + 9);
+	priv->block_device = g_strdup (device);
+	BRASERO_MEDIA_LOG ("Initializing block drive %s", priv->block_device);
+#else
+	priv->device = g_strdup (device);
+#endif
 
-	BRASERO_MEDIA_LOG ("Initializing drive %s", priv->block_path);
+	BRASERO_MEDIA_LOG ("Initializing drive %s from device", priv->device);
 
 	/* NOTE: why a thread? Because in case of a damaged medium, brasero can
 	 * block on some functions until timeout and if we do this in the main
 	 * thread then our whole UI blocks. This medium won't be exported by the
 	 * BraseroDrive that exported until it returns PROBED signal.
 	 * One (good) side effect is that it also improves start time. */
+	g_mutex_lock (priv->mutex);
 	priv->probe = g_thread_create (brasero_drive_probe_thread,
 				       drive,
-				       TRUE,
+				       FALSE,
 				       NULL);
+	g_mutex_unlock (priv->mutex);
 }
 
 static void
@@ -938,6 +1296,7 @@ brasero_drive_set_property (GObject *object,
 			    GParamSpec *pspec)
 {
 	BraseroDrivePrivate *priv;
+	GDrive *gdrive = NULL;
 
 	g_return_if_fail (BRASERO_IS_DRIVE (object));
 
@@ -948,15 +1307,21 @@ brasero_drive_set_property (GObject *object,
 	case PROP_UDI:
 		break;
 	case PROP_GDRIVE:
-		priv->gdrive = g_value_get_object (value);
-		if (!priv->gdrive) {
-			priv->probed = TRUE;
+		if (!priv->device)
+			break;
+
+		gdrive = g_value_get_object (value);
+		brasero_drive_update_gdrive (BRASERO_DRIVE (object), gdrive);
+		break;
+	case PROP_DEVICE:
+		if (!g_value_get_string (value)) {
 			priv->medium = g_object_new (BRASERO_TYPE_VOLUME,
 						     "drive", object,
 						     NULL);
+			priv->probed = TRUE;
 		}
 		else
-			brasero_drive_init_real (BRASERO_DRIVE (object));
+			brasero_drive_init_real_device (BRASERO_DRIVE (object), g_value_get_string (value));
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -983,12 +1348,27 @@ brasero_drive_get_property (GObject *object,
 	case PROP_GDRIVE:
 		g_value_set_object (value, priv->gdrive);
 		break;
+	case PROP_DEVICE:
+		g_value_set_string (value, priv->device);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
 	}
 }
 
+static void
+brasero_drive_init (BraseroDrive *object)
+{
+	BraseroDrivePrivate *priv;
+
+	priv = BRASERO_DRIVE_PRIVATE (object);
+	priv->cancel = g_cancellable_new ();
+
+	priv->mutex = g_mutex_new ();
+	priv->cond = g_cond_new ();
+}
+
 static void
 brasero_drive_finalize (GObject *object)
 {
@@ -996,27 +1376,59 @@ brasero_drive_finalize (GObject *object)
 
 	priv = BRASERO_DRIVE_PRIVATE (object);
 
+	BRASERO_MEDIA_LOG ("Finalizing BraseroDrive");
+
+	g_mutex_lock (priv->mutex);
 	if (priv->probe) {
 		priv->probe_cancelled = TRUE;
-		g_thread_join (priv->probe);
-		priv->probe = 0;
+		g_cond_wait (priv->cond, priv->mutex);
 	}
+	g_mutex_unlock (priv->mutex);
 
 	if (priv->probe_id) {
 		g_source_remove (priv->probe_id);
 		priv->probe_id = 0;
 	}
 
-	if (priv->block_path) {
-		g_free (priv->block_path);
-		priv->block_path = NULL;
+	if (priv->mutex) {
+		g_mutex_free (priv->mutex);
+		priv->mutex = NULL;
+	}
+
+	if (priv->cond) {
+		g_cond_free (priv->cond);
+		priv->cond = NULL;
 	}
 
 	if (priv->medium) {
+		g_signal_emit (object,
+			       drive_signals [MEDIUM_REMOVED],
+			       0,
+			       priv->medium);
 		g_object_unref (priv->medium);
 		priv->medium = NULL;
 	}
 
+	if (priv->name) {
+		g_free (priv->name);
+		priv->name = NULL;
+	}
+
+	if (priv->device) {
+		g_free (priv->device);
+		priv->device = NULL;
+	}
+
+	if (priv->block_device) {
+		g_free (priv->block_device);
+		priv->block_device = NULL;
+	}
+
+	if (priv->udi) {
+		g_free (priv->udi);
+		priv->udi = NULL;
+	}
+
 	if (priv->gdrive) {
 		g_signal_handlers_disconnect_by_func (priv->gdrive,
 						      brasero_drive_medium_gdrive_changed_cb,
@@ -1084,7 +1496,7 @@ brasero_drive_class_init (BraseroDriveClass *klass)
 	g_object_class_install_property (object_class,
 	                                 PROP_UDI,
 	                                 g_param_spec_string("udi",
-	                                                     "Deprecated",
+	                                                     "udi",
 	                                                     "HAL udi as a string (Deprecated)",
 	                                                     NULL,
 	                                                     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
@@ -1094,6 +1506,13 @@ brasero_drive_class_init (BraseroDriveClass *klass)
 	                                                      "GDrive",
 	                                                      "A GDrive object for the drive",
 	                                                      G_TYPE_DRIVE,
-	                                                     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	                                                     G_PARAM_READWRITE));
+	g_object_class_install_property (object_class,
+	                                 PROP_DEVICE,
+	                                 g_param_spec_string ("device",
+	                                                      "Device",
+	                                                      "Device path for the drive",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 }
 
