@@ -85,6 +85,8 @@ typedef struct _BraseroMediumPrivate BraseroMediumPrivate;
 struct _BraseroMediumPrivate
 {
 	GThread *probe;
+	GMutex *mutex;
+	GCond *cond;
 	gint probe_id;
 
 	GSList *tracks;
@@ -831,9 +833,81 @@ brasero_medium_get_capacity (BraseroMedium *medium,
 }
 
 /**
- * Test presence of simulate burning
+ * Test presence of simulate burning/ SAO/ DAO
  */
 
+static gboolean
+brasero_medium_set_write_mode_page_tao (BraseroMedium *self,
+                                        BraseroDeviceHandle *handle,
+                                        BraseroScsiErrCode *code)
+{
+	BraseroScsiModeData *data = NULL;
+	BraseroScsiWritePage *wrt_page;
+	BraseroMediumPrivate *priv;
+	BraseroScsiResult result;
+	int size;
+
+	BRASERO_MEDIA_LOG ("Setting write mode page");
+
+	priv = BRASERO_MEDIUM_PRIVATE (self);
+
+	/* NOTE: this works for CDR, DVDR+-, BDR-SRM */
+	/* make sure the current write mode is TAO. Otherwise the drive will
+	 * return the first sector of the pregap instead of the first user
+	 * accessible sector. */
+	result = brasero_spc1_mode_sense_get_page (handle,
+						   BRASERO_SPC_PAGE_WRITE,
+						   &data,
+						   &size,
+						   code);
+	if (result != BRASERO_SCSI_OK) {
+		BRASERO_MEDIA_LOG ("MODE SENSE failed");
+		/* This isn't necessarily a problem! we better try the rest */
+		return FALSE;
+	}
+
+	wrt_page = (BraseroScsiWritePage *) &data->page;
+
+	BRASERO_MEDIA_LOG ("Former write type %d", wrt_page->write_type);
+	BRASERO_MEDIA_LOG ("Former track mode %d", wrt_page->track_mode);
+	BRASERO_MEDIA_LOG ("Former data block type %d", wrt_page->data_block_type);
+
+	/* "reset some stuff to be on the safe side" (words and ideas
+	 * taken from k3b:)). */
+	wrt_page->ps = 0;
+	wrt_page->BUFE = 0;
+	wrt_page->multisession = 0;
+	wrt_page->testwrite = 0;
+	wrt_page->LS_V = 0;
+	wrt_page->copy = 0;
+	wrt_page->FP = 0;
+	wrt_page->session_format = 0;
+	BRASERO_SET_16 (wrt_page->pause_len, 150);
+
+	if (priv->info & BRASERO_MEDIUM_CD) {
+		wrt_page->write_type = BRASERO_SCSI_WRITE_TAO;
+		wrt_page->track_mode = 4;
+	}
+	else if (priv->info & BRASERO_MEDIUM_DVD) {
+		wrt_page->write_type = BRASERO_SCSI_WRITE_PACKET_INC;
+		wrt_page->track_mode = 5;
+	}
+
+	wrt_page->data_block_type = 8;
+
+	result = brasero_spc1_mode_select (handle, data, size, code);
+	g_free (data);
+
+	if (result != BRASERO_SCSI_OK) {
+		BRASERO_MEDIA_LOG ("MODE SELECT failed");
+
+		/* This isn't necessarily a problem! we better try */
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
 static gboolean
 brasero_medium_test_CD_TAO_simulate (BraseroMedium *self,
 				     BraseroDeviceHandle *handle,
@@ -1759,79 +1833,6 @@ brasero_medium_track_set_leadout_CDR_blank (BraseroMedium *self,
 	return TRUE;
 }
 
-static gboolean
-brasero_medium_set_write_mode_page (BraseroMedium *self,
-				    BraseroDeviceHandle *handle,
-				    BraseroScsiErrCode *code)
-{
-	BraseroScsiModeData *data = NULL;
-	BraseroScsiWritePage *wrt_page;
-	BraseroMediumPrivate *priv;
-	BraseroScsiResult result;
-	int size;
-
-	BRASERO_MEDIA_LOG ("Setting write mode page");
-
-	priv = BRASERO_MEDIUM_PRIVATE (self);
-
-	/* NOTE: this works for CDR, DVDR+-, BDR-SRM */
-	/* make sure the current write mode is TAO. Otherwise the drive will
-	 * return the first sector of the pregap instead of the first user
-	 * accessible sector. */
-	result = brasero_spc1_mode_sense_get_page (handle,
-						   BRASERO_SPC_PAGE_WRITE,
-						   &data,
-						   &size,
-						   code);
-	if (result == BRASERO_SCSI_OK) {
-		wrt_page = (BraseroScsiWritePage *) &data->page;
-
-		BRASERO_MEDIA_LOG ("Former write type %d", wrt_page->write_type);
-		BRASERO_MEDIA_LOG ("Former track mode %d", wrt_page->track_mode);
-		BRASERO_MEDIA_LOG ("Former data block type %d", wrt_page->data_block_type);
-
-		/* "reset some stuff to be on the safe side" (words and ideas
-		 * taken from k3b:)). */
-		wrt_page->ps = 0;
-		wrt_page->BUFE = 0;
-		wrt_page->multisession = 0;
-		wrt_page->testwrite = 0;
-		wrt_page->LS_V = 0;
-		wrt_page->copy = 0;
-		wrt_page->FP = 0;
-		wrt_page->session_format = 0;
-		BRASERO_SET_16 (wrt_page->pause_len, 150);
-
-		if (priv->info & BRASERO_MEDIUM_CD) {
-			wrt_page->write_type = BRASERO_SCSI_WRITE_TAO;
-			wrt_page->track_mode = 4;
-		}
-		else if (priv->info & BRASERO_MEDIUM_DVD) {
-			wrt_page->write_type = BRASERO_SCSI_WRITE_PACKET_INC;
-			wrt_page->track_mode = 5;
-		}
-
-		wrt_page->data_block_type = 8;
-
-		result = brasero_spc1_mode_select (handle, data, size, code);
-		g_free (data);
-
-		if (result != BRASERO_SCSI_OK) {
-			BRASERO_MEDIA_LOG ("MODE SELECT failed");
-
-			/* This isn't necessarily a problem! we better try */
-			return FALSE;
-		}
-	}
-	else {
-		BRASERO_MEDIA_LOG ("MODE SENSE failed");
-		/* This isn't necessarily a problem! we better try the rest */
-		return FALSE;
-	}
-
-	return TRUE;
-}
-
 static gboolean
 brasero_medium_track_set_leadout (BraseroMedium *self,
 				  BraseroDeviceHandle *handle,
@@ -1859,7 +1860,7 @@ brasero_medium_track_set_leadout (BraseroMedium *self,
 		 * Carry on even if it fails.
 		 * This can work with CD-R/W and DVD-R/W. + media don't use the
 		 * write mode page anyway. */
-		result = brasero_medium_set_write_mode_page (self, handle, code);
+		result = brasero_medium_set_write_mode_page_tao (self, handle, code);
 		if (result == FALSE
 		&&  BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_CDR|BRASERO_MEDIUM_BLANK))
 			return brasero_medium_track_set_leadout_CDR_blank (self,
@@ -2324,9 +2325,10 @@ brasero_medium_get_medium_type (BraseroMedium *self,
 
 		/* If this fails it means that this drive is probably older than
 		 * MMC1 spec or does not conform to it. */
-		if (result != TRUE)
-		if (result != TRUE)
+		if (result != TRUE) {
+			priv->info = BRASERO_MEDIUM_NONE;
 			return FALSE;
+		}
 
 		/* The only thing here left to determine is if that's a WRITABLE
 		 * or a REWRITABLE. To determine that information, we need to
@@ -2370,6 +2372,10 @@ brasero_medium_get_medium_type (BraseroMedium *self,
 	}
 
 	switch (profile) {
+	case BRASERO_SCSI_PROF_EMPTY:
+		priv->info = BRASERO_MEDIUM_NONE;
+		return FALSE;
+
 	case BRASERO_SCSI_PROF_CDROM:
 		priv->info = BRASERO_MEDIUM_CDROM;
 		priv->type = types [1];
@@ -2856,7 +2862,6 @@ brasero_medium_init_real (BraseroMedium *object,
 		return;
 
 	result = brasero_medium_get_medium_type (object, handle, &code);
-	if (result != TRUE)
 	if (result != TRUE)
 		return;
 
@@ -2864,7 +2869,6 @@ brasero_medium_init_real (BraseroMedium *object,
 		return;
 
 	result = brasero_medium_get_speed (object, handle, &code);
-	if (result != TRUE)
 	if (result != TRUE)
 		return;
 
@@ -2875,6 +2879,10 @@ brasero_medium_init_real (BraseroMedium *object,
 	if (priv->probe_cancelled)
 		return;
 
+	brasero_medium_init_caps (object, handle, &code);
+	if (priv->probe_cancelled)
+		return;
+
 	result = brasero_medium_get_contents (object, handle, &code);
 	if (result != TRUE)
 	if (result != TRUE)
@@ -2891,10 +2899,6 @@ brasero_medium_init_real (BraseroMedium *object,
 	if (priv->probe_cancelled)
 		return;
 
-	brasero_medium_init_caps (object, handle, &code);
-	if (priv->probe_cancelled)
-		return;
-
 	/* read CD-TEXT title */
 	if (priv->info & BRASERO_MEDIUM_HAS_AUDIO)
 		brasero_medium_read_CD_TEXT (object, handle, &code);
@@ -2929,10 +2933,11 @@ brasero_medium_probed (gpointer data)
 {
 	BraseroMediumPrivate *priv;
 
+	g_return_val_if_fail (BRASERO_IS_MEDIUM (data), FALSE);
+
 	priv = BRASERO_MEDIUM_PRIVATE (data);
 
-	g_thread_join (priv->probe);
-	priv->probe = NULL;
+	priv->probe_id = 0;
 
 	/* This signal must be emitted in the main thread */
 	GDK_THREADS_ENTER ();
@@ -2941,7 +2946,6 @@ brasero_medium_probed (gpointer data)
 		       0);
 	GDK_THREADS_LEAVE ();
 
-	priv->probe_id = 0;
 	return FALSE;
 }
 
@@ -2967,19 +2971,15 @@ brasero_medium_probe_thread (gpointer self)
 	while (!handle && counter <= BRASERO_MEDIUM_OPEN_ATTEMPTS) {
 		sleep (1);
 
-		if (priv->probe_cancelled) {
-			priv->probe = NULL;
-			return NULL;
-		}
+		if (priv->probe_cancelled)
+			goto end;
 
 		counter ++;
 		handle = brasero_device_handle_open (device, FALSE, &code);
 	}
 
-	if (priv->probe_cancelled) {
-		priv->probe = NULL;
-		return NULL;
-	}
+	if (priv->probe_cancelled)
+		goto end;
 
 	if (handle) {
 		BRASERO_MEDIA_LOG ("Open () succeeded");
@@ -2987,20 +2987,24 @@ brasero_medium_probe_thread (gpointer self)
 		/* NOTE: if we wanted to know the status we'd need to read the 
 		 * error code variable which is currently NULL */
 		while (brasero_spc1_test_unit_ready (handle, &code) != BRASERO_SCSI_OK) {
-			if (code != BRASERO_SCSI_NOT_READY) {
-				priv->probe = NULL;
+			if (code == BRASERO_SCSI_NO_MEDIUM) {
+				brasero_device_handle_close (handle);
+				BRASERO_MEDIA_LOG ("No medium inserted");
+				priv->info = BRASERO_MEDIUM_NONE;
+				goto end;
+			}
+			else if (code != BRASERO_SCSI_NOT_READY) {
 				brasero_device_handle_close (handle);
 				BRASERO_MEDIA_LOG ("Device does not respond");
-				break;
+				goto end;
 			}
 
 			sleep (2);
 
 			if (priv->probe_cancelled) {
-				priv->probe = NULL;
 				brasero_device_handle_close (handle);
 				BRASERO_MEDIA_LOG ("Device probing cancelled");
-				return NULL;
+				goto end;
 			}
 		}
 
@@ -3012,7 +3016,18 @@ brasero_medium_probe_thread (gpointer self)
 	else
 		BRASERO_MEDIA_LOG ("Open () failed: medium busy");
 
-	priv->probe_id = g_idle_add (brasero_medium_probed, self);
+end:
+
+	if (!priv->probe_cancelled)
+		priv->probe_id = g_idle_add (brasero_medium_probed, self);
+
+	g_mutex_lock (priv->mutex);
+	priv->probe = NULL;
+	g_cond_broadcast (priv->cond);
+	g_mutex_unlock (priv->mutex);
+
+	g_thread_exit (0);
+
 	return NULL;
 }
 
@@ -3028,10 +3043,12 @@ brasero_medium_probe (BraseroMedium *self)
 	 * thread then our whole UI blocks. This medium won't be exported by the
 	 * BraseroDrive that exported until it returns PROBED signal.
 	 * One (good) side effect is that it also improves start time. */
+	g_mutex_lock (priv->mutex);
 	priv->probe = g_thread_create (brasero_medium_probe_thread,
 				       self,
-				       TRUE,
+				       FALSE,
 				       NULL);
+	g_mutex_unlock (priv->mutex);
 }
 
 static void
@@ -3053,6 +3070,9 @@ brasero_medium_init (BraseroMedium *object)
 	priv = BRASERO_MEDIUM_PRIVATE (object);
 	priv->next_wr_add = -1;
 
+	priv->mutex = g_mutex_new ();
+	priv->cond = g_cond_new ();
+
 	/* we can't do anything here since properties haven't been set yet */
 }
 
@@ -3065,17 +3085,28 @@ brasero_medium_finalize (GObject *object)
 
 	BRASERO_MEDIA_LOG ("Finalizing Medium object");
 
+	g_mutex_lock (priv->mutex);
 	if (priv->probe) {
 		priv->probe_cancelled = TRUE;
-		g_thread_join (priv->probe);
-		priv->probe = 0;
+		g_cond_wait (priv->cond, priv->mutex);
 	}
+	g_mutex_unlock (priv->mutex);
 
 	if (priv->probe_id) {
 		g_source_remove (priv->probe_id);
 		priv->probe_id = 0;
 	}
 
+	if (priv->mutex) {
+		g_mutex_free (priv->mutex);
+		priv->mutex = NULL;
+	}
+
+	if (priv->cond) {
+		g_cond_free (priv->cond);
+		priv->cond = NULL;
+	}
+
 	if (priv->id) {
 		g_free (priv->id);
 		priv->id = NULL;
