@@ -578,8 +578,8 @@ brasero_audio_disc_add_uri_real (BraseroAudioDisc *disc,
 				 GtkTreePath **path_return)
 {
 	BraseroTrackStreamCfg *track;
+	BraseroTrack *sibling = NULL;
 	BraseroSessionCfg *session;
-	BraseroTrack *sibling;
 	GtkTreeModel *store;
 
 	g_return_val_if_fail (uri != NULL, BRASERO_DISC_ERROR_UNKNOWN);
@@ -661,7 +661,7 @@ brasero_audio_disc_video_file_dialog (BraseroAudioDisc *disc,
 	answer = gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
 
-	return (answer == GTK_RESPONSE_OK);
+	return (answer != GTK_RESPONSE_OK);
 }
 
 static void
@@ -881,6 +881,7 @@ brasero_audio_disc_session_changed (BraseroSessionCfg *session,
 			brasero_burn_session_remove_track (BRASERO_BURN_SESSION (session),
 							   BRASERO_TRACK (track));
 			g_free (uri);
+			continue;
 		}
 
 		if (BRASERO_STREAM_FORMAT_HAS_VIDEO (format)) {
@@ -991,20 +992,30 @@ brasero_audio_disc_set_session_contents (BraseroDisc *disc,
 					 BraseroBurnSession *session)
 {
 	BraseroAudioDisc *audio;
+	GtkTreeModel *current_model;
 	BraseroVideoTreeModel *model;
 
 	audio = BRASERO_AUDIO_DISC (disc);
 
+	if (audio->priv->add_dir)
+		brasero_io_cancel_by_base (audio->priv->add_dir);
+
+	if (audio->priv->add_playlist)
+		brasero_io_cancel_by_base (audio->priv->add_playlist);
+
+	/* disconnect some signals */
+	current_model = gtk_tree_view_get_model (GTK_TREE_VIEW (audio->priv->tree));
+	if (current_model) {
+		BraseroSessionCfg *current_session;
+
+		current_session = brasero_video_tree_model_get_session (BRASERO_VIDEO_TREE_MODEL (current_model));
+		if (current_session)
+			g_signal_handlers_disconnect_by_func (current_session,
+							      brasero_audio_disc_session_changed,
+							      disc);
+	}
+
 	if (!session) {
-		GtkTreeModel *model;
-		BraseroSessionCfg *session;
-
-		model = gtk_tree_view_get_model (GTK_TREE_VIEW (audio->priv->tree));
-		session = brasero_video_tree_model_get_session (BRASERO_VIDEO_TREE_MODEL (model));
-		g_signal_handlers_disconnect_by_func (session,
-						      brasero_audio_disc_session_changed,
-						      disc);
-
 		gtk_tree_view_set_model (GTK_TREE_VIEW (audio->priv->tree), NULL);
 		return BRASERO_DISC_OK;
 	}
